 
/*
* Joystick-rgbDiode.ino
* 
* This is a demo sketch for ftDuinoBlue. It allows you to create a custom
* remote android user interface for your project right inside you arduino
* sketch. No android programming required.
* 
* For more info see http://ftduino.de/blue 
*/

// use i2c bluetooth adapter
#include "ftduinoblue.h"
#include "I2cSerialBt.h"
I2cSerialBt btSerial;
#include <Ftduino.h>

// the layout to be sent to the ftDuinoBlue app
const char layout[] PROGMEM = 
"<layout orientation='portrait' name='Joystick + RGB-Diode'>"
"<switch id='1' size='20' width='parent' place='hcenter;top'>LED on/off</switch>"
"<label id='2' size='20' place='left;below:1'>rot</label>"
"<slider id='3' width='parent' max='255' place='hcenter;below:2'/>"
"<label id='4' size='20' place='left;below:3'>grün</label>"
"<slider id='5' width='parent' max='255' place='hcenter;below:4'/>"
"<label id='6' size='20' place='left;below:5'>blau</label>"
"<slider id='7' width='parent' max='255' place='hcenter;below:6'/>"
"<space id='8' width='20' height='30' place='hcenter;below:7'/>"
"<label id='9' size='20' place='hcenter;below:8'></label>"
"<space id='10' width='20' height='30' place='hcenter;below:9'/>"
"<joystick id='11' bgcolor = 'red' width='280' height='280' place='hcenter;below:10'/>"
"</layout>";

FtduinoBlue ftdblue(btSerial, layout);

void setup() {
ftduino.init();
ftduino.output_set(Ftduino::O8, Ftduino::HI, 32);
ftduino.output_set(Ftduino::O2, Ftduino::HI, 32);
ftduino.output_set(Ftduino::O1, Ftduino::HI, 32);

// register callback for ftduinoblue
ftdblue.setCallback(ftduinoblue_callback);

// wait max 1 sec for adapter to appear on bus. This is not
// needed as begin() will wait for the device. But this way
// we can use the led as an inidictaor for problems with 
// the i2c uart adapater
if(!btSerial.check(1000)) {
// fast blink with led on failure
pinMode(LED_BUILTIN, OUTPUT); 
while(true) {
digitalWrite(LED_BUILTIN, HIGH);
delay(100);
digitalWrite(LED_BUILTIN, LOW);
delay(100);
}}

// initialize i2c uart to 9600 baud
btSerial.begin(9600);

// prepare builtin-led
pinMode(LED_BUILTIN, OUTPUT);
digitalWrite(LED_BUILTIN, LOW); 
};

// led config. This has a startup default and can be changed
// by the user via bluetooth. This is also reported back to
// the user app to provide the correct initial values should
// the user reconnect with the app
static char ledChanged = true;
static char ledState = true; // LED is on
static uint8_t ledBlinkSpeed = 50, // ~ 2 Hz
   redBrightness = 128, // 50% brightness
   greenBrightness = 128, // 50% brightness
   blueBrightness = 128; // 50% brightness
static bool vw = true;
static int m2speed = 0, m3speed = 0, // maximum 64
   xs = 0, ys = 0;
static String st;
  
void ftduinoblue_callback(struct FtduinoBlue::reply *r) { 
switch(r->type) {
case FtduinoBlue::FTDB_STATE:
  ftdblue.print("SWITCH 1 ");
  ftdblue.println(ledState?"ON":"OFF");
  ftdblue.print("SLIDER 3 ");
  ftdblue.println(redBrightness);
  ftdblue.print("SLIDER 5 ");
  ftdblue.println(greenBrightness); 
  ftdblue.print("SLIDER 7 ");
  ftdblue.println(blueBrightness); 
break;

case FtduinoBlue::FTDB_BUTTON:
break;

case FtduinoBlue::FTDB_SWITCH:

// make sure the led reacts on switch 1
if(r->id == 1) {
ledChanged = true;
ledState = r->state; 
}
break;

case FtduinoBlue::FTDB_SLIDER:
  if(r->id == 3) {
  redBrightness = r->slider;
  ftduino.output_set(Ftduino::O8, Ftduino::HI, redBrightness/4);
}
if(r->id == 5) {
greenBrightness = r->slider;
ftduino.output_set(Ftduino::O2, Ftduino::HI, greenBrightness/4);
}
if(r->id == 7) {
blueBrightness = r->slider;
ftduino.output_set(Ftduino::O1, Ftduino::HI, blueBrightness/4);
}
break;

case FtduinoBlue::FTDB_JOYSTICK:
  xs = (r->joystick.x) * 0.35 ;
  ys = (r->joystick.y)* 0.68 ;
break;

default:
break;
}}
void motoren (bool vor, int m2, int m3){
  if (vor) {
      ftduino.motor_set(Ftduino::M2, Ftduino::RIGHT, m2);
      ftduino.motor_set(Ftduino::M3, Ftduino::RIGHT, m3);}
  else {ftduino.motor_set(Ftduino::M2, Ftduino::LEFT, m2);
      ftduino.motor_set(Ftduino::M3, Ftduino::LEFT, m3);     
}}
void loop() {
if (ys < 0) vw = false; else vw = true;
 if (vw) {
    m3speed = ys + xs;
    m2speed = ys - xs;
 }
 else {
    m3speed = xs - ys;
    m2speed = (-1)*ys - xs;
 }
 if (m2speed > 64) m2speed = 64;
 if (m2speed < 13) {if (m2speed > 1) m2speed = 12; else m2speed = 0;}
 if (m3speed > 64) m3speed = 64;
 if (m3speed < 13) {if (m3speed > 1) m3speed = 12; else m3speed = 0;}
 
 st = " " + String(m2speed) + " || " + String(m3speed)  ;
 ftdblue.print("TEXT "); ftdblue.print(9, DEC); ftdblue.println(st);
 // Kontroll-Anzeige der Motorwerte in Label 9
 motoren(vw,m2speed,m3speed);

// led blink timer
static uint32_t led_tick = 0;
static bool led_state = false;

// led is updated on timer or by user change
if(ledChanged || (millis() - led_tick) > 50+10*(100-ledBlinkSpeed)) {
// check if led update was triggered by user
if(!ledChanged) {
// no. So it was the timer -> handle timer
led_state = !led_state;
led_tick = millis();
} else
ledChanged = false;

// update state of physical led. Set analog brightness if
// a) led is enabled by user and b) led is actually in the on state
// of the blinking. Switch it off otherwise
if(led_state && ledState) analogWrite(LED_BUILTIN, 200);
else digitalWrite(LED_BUILTIN, LOW); 
}
ftdblue.handle();
}; 
